@@ -1,4 +1,4 @@
-from sympy import S, diff, lambdify, symbols, sqrt, cos,sin, numbered_symbols, simplify,binomial, hyper, hyperexpand, Function, factorial,elliptic_k,elliptic_e, expand_trig
+from sympy import S, diff, lambdify, symbols, sqrt, cos,sin, numbered_symbols, simplify,binomial, hyper, hyperexpand, Function, factorial,elliptic_k,elliptic_e, expand_trig, Function,bell
 from sympy import I,exp,series
 from . import clibcelmech
 from ctypes import Structure, c_double, POINTER, c_float, c_int, c_uint, c_uint32, c_int64, c_long, c_ulong, c_ulonglong, c_void_p, c_char_p, CFUNCTYPE, byref, create_string_buffer, addressof, pointer, cast
@@ -10,40 +10,142 @@ from scipy.special import poch,factorial2,binom,factorial,gamma,hyp2f1
 from collections import defaultdict
 import warnings
 
-def get_DFCoeff_symbol(k1,k2,k3,k4,k5,k6,z1,z2,z3,z4,indexIn,indexOut):
-    return symbols("C_{0}\,{1}\,{2}\,{3}\,{4}\,{5}^{6}\,{7}\,{8}\,{9};({10}\,{11})".format(
-        k1,k2,k3,k4,k5,k6,z1,z2,z3,z4,indexIn,indexOut)
-    )
+def get_df_term_latex(k1,k2,k3,k4,k5,k6,nu1,nu2,nu3,nu4,l1,l2,indexIn,indexOut):
+    r"""
+    Get the latex expression for the disturbing function coefficient
+
+    .. math::
+        C_{\pmb{k}}^{\pmb{\nu},\pmb{l}}(\alpha_{i,j})
+
+    Returns
+    -------
+    latex
+    """
+    assert l1 + l2 < 2, "I'm not going to print something so ridiculuous!"
+    C  = r"\tilde{{C}}_{{({0},{1},{2},{3},{4},{5})}}".format(k1,k2,k3,k4,k5,k6)
+    if l1==1:
+        C = r"\alpha\frac{d}{d\alpha}" + C
+    if l2==1:
+        C = r"\alpha\frac{d}{d\alpha}" + C
+    C += r"^{{({0},{1},{2},{3})}}".format(nu1,nu2,nu3,nu4)
+    C += r"(\alpha_{{{0},{1}}})".format(indexIn,indexOut)
+    term = r"{0}\frac{{Gm_{1}m_{2}}}{{a_{{{2},0}}}}".format(["-","+"][l2],indexIn,indexOut)
+    term += C
+    e1exp = abs(k3) + 2*nu3
+    e2exp = abs(k4) + 2*nu4
+    i1exp = abs(k5) + 2*nu1
+    i2exp = abs(k6) + 2*nu2
+    if l1==1:
+        term+=r"\left(\frac{{\delta a_{0}}}{{a_{{{0},0}}}}\right)".format(indexIn)
+    if l2==1:
+        term+=r"\left(\frac{{\delta a_{0}}}{{a_{{{0},0}}}}\right)".format(indexOut)
+    if e1exp > 0:
+        if e1exp == 1:
+            term += r"e_{0}".format(indexIn)
+        else:
+            term += r"e_{0}^{1}".format(indexIn, e1exp)
+    if e2exp > 0:
+        if e2exp == 1:
+            term += r"e_{0}".format(indexOut)
+        else:
+            term += r"e_{0}^{1}".format(indexOut, e2exp)
+    if i1exp > 0:
+        if i1exp == 1:
+            term += r"i_{0}".format(indexIn)
+        else:
+            term += r"i_{0}^{1}".format(indexIn, i1exp)
+    if i2exp > 0:
+        if i2exp == 1:
+            term += r"i_{0}".format(indexOut)
+        else:
+            term += r"i_{0}^{1}".format(indexOut, i2exp)
+    arg = r""
+    if k1 == 0 and k2 == 0 and k3 == 0 and k4 == 0 and k5==0 and k6 == 0:
+        pass # don't write cosine
+    else:
+        if k1 < 0:
+            arg += r"-"
+        if abs(k1) > 1:
+            arg += r"{0}".format(abs(k1))
+        if k1 != 0:
+            arg += r"\lambda_{0}".format(indexOut)
+        if k2 != 0:
+            arg += r"+" if k2 > 0 else r"-"
+            if abs(k2) > 1:
+                arg += r"{0}".format(abs(k2))
+            arg += r"\lambda_{0}".format(indexIn)
+        if k3 != 0:
+            arg += r"+" if k3 > 0 else r"-"
+            if abs(k3) > 1:
+                arg += r"{0}".format(abs(k3))
+            arg += r"\varpi_{0}".format(indexIn)  
+        if k4 != 0:
+            arg += r"+" if k4 > 0 else r"-"
+            if abs(k4) > 1:
+                arg += r"{0}".format(abs(k4))
+            arg += r"\varpi_{0}".format(indexOut)
+        if k5 != 0:
+            arg += r"+" if k5 > 0 else r"-"
+            if abs(k5) > 1:
+                arg += r"{0}".format(abs(k5))
+            arg += r"\Omega_{0}".format(indexIn)
+        if k6 != 0:
+            arg += r"+" if k6 > 0 else r"-"
+            if abs(k6) > 1:
+                arg += r"{0}".format(abs(k6))
+            arg += r"\Omega_{0}".format(indexOut)
+        term += r"\cos({0})".format(arg)
+        
+    return term
+
+def get_df_coefficient_symbol(k1,k2,k3,k4,k5,k6,nu1,nu2,nu3,nu4,l1,l2,indexIn,indexOut):
+    r"""
+    Get a sympy symbol for the disturbing function coefficient
+
+    .. math::
+        C_{\pmb{k}}^{\pmb{\nu},\pmb{l}}(\alpha_{i,j})
+
+    Returns
+    -------
+    sympy symbol
+    """
+    symbol_str  = r"C_{{({0}\,{1}\,{2}\,{3}\,{4}\,{5})}}".format(k1,k2,k3,k4,k5,k6)
+    symbol_str += r"^{{({0}\,{1}\,{2}\,{3})\,({4}\,{5})}}".format(nu1,nu2,nu3,nu4,l1,l2)
+    symbol_str += r"(\alpha_{{{0}\,{1}}})".format(indexIn,indexOut)
+    return symbols(symbol_str)
 
 def _delta(*args):
+    """
+    Return 0 if all arguments are 0, otherwise return 1.
+    """
     intarr  = np.array([args],dtype=np.int64)
     if np.alltrue(intarr == 0):
         return 0
     else:
         return 1
     
-def DFArguments_dictionary(Nmax):
-    """
+def df_arguments_dictionary(Nmax):
+    r"""
     Get the arguments appearing in the disturbing function
-    up to order Nmax.  
+    up to order Nmax. Arguments are returned as a nested dictionary.
+    The outer level keys are orders. 
+    The inner level keys are 
+    :math:`\Delta k = |k_1-k_2|`, 
+    denoting the order of MMR for which the argument appears. 
+    The values of the inner level dictionaries are lists of
+    tuples containing :math:`(k_3,k_4,k_5,k_6)`.
+    Specifically, the dictionary is returned in the form:
     
-    Arguments are returned as a nested dictionary. 
-    The outer level keys are orders. The inner level
-    keys are dj = |j1-j2|, denoting the order of MMR
-    for which the argument appears. The values of the 
-    inner level dictionaries are lists of 
-    tuples containing (j3,j4,j5,j6). 
-    
-    {
-        0:{0:[(0,0,0,0)],
-        1:{1:[(-1,0,0,0),(0,-1,0,0)]},
-        ...
-        Nmax:{
-            0:[(j3,j4,j5,j6)],
-            ...
-            Nmax:[(j3,j4,j5,j6),...]
-        }
-    }
+    | {
+    |     0:{0:[(0,0,0,0)],
+    |     1:{1:[(-1,0,0,0),(0,-1,0,0)]},
+    |     ...
+    |     Nmax:{
+    |         0:[(k3,k4,k5,k6)],
+    |         ...
+    |         Nmax:[(k3,k4,k5,k6),...]
+    |     }
+    | }
     
     Arguments
     ---------
@@ -71,104 +173,142 @@ def DFArguments_dictionary(Nmax):
                 s1_lo = -1 * s1_hi * _delta(h,k,s)
                 for s1 in range(s1_lo,s1_hi + 1):
                     dj = 2 * h + s + s1
-                    sgn = 1 if dj is 0 else np.sign(dj)
-                    j3=-sgn*s
-                    j4=-sgn*s1
-                    j5=-sgn*(hplusk)
-                    j6=-sgn*(hminusk)
-                    N = abs(j3) + abs(j4) + abs(j5) + abs(j6)
-                    args_dict[N][dj*sgn].append((j3,j4,j5,j6))
+                    sgn = 1 if dj == 0 else np.sign(dj)
+                    k3=-sgn*s
+                    k4=-sgn*s1
+                    k5=-sgn*(hplusk)
+                    k6=-sgn*(hminusk)
+                    N = abs(k3) + abs(k4) + abs(k5) + abs(k6)
+                    args_dict[N][dj*sgn].append((k3,k4,k5,k6))
     return args_dict
 
-def _zcombos_iter(ztot):
-    for z1 in range(ztot+1):
-        for z2 in range(ztot+1-z1):
-            for z3 in range(ztot+1-z1-z2):
-                z4 = ztot - z1 - z2 - z3
-                yield (z1,z2,z3,z4)
-                
-def ResonanceTermsList(j,k,Nmin,Nmax):
+def _nucombos(nutot):
+    nucombos = []
+    for nu1 in range(nutot+1):
+        for nu2 in range(nutot+1-nu1):
+            for nu3 in range(nutot+1-nu1-nu2):
+                nu4 = nutot - nu1 - nu2 - nu3
+                nucombos.append((nu1, nu2, nu3, nu4))
+    return nucombos
+
+def _lcombos(ltot):
+    lcombos = []
+    for l1 in range(ltot+1):
+        l2 = ltot - l1
+        lcombos.append((l1, l2))
+    return lcombos
+def k_nu_depend_on_inclinations(k,nu):
+    _,_,_,_,k5,k6 = k
+    nu1,nu2,_,_ = nu
+    arr=np.array([k5,k6,nu1,nu2])
+    return np.any(arr!=0)
+def k_nu_depend_on_eccentricities(k,nu):
+    _,_,k3,k4,_,_ = k
+    _,_,nu3,nu4 = nu
+    arr=np.array([k3,k4,nu3,nu4])
+    return np.any(arr!=0)
+def list_resonance_terms(p,q,min_order=None,max_order=None,eccentricities=True,inclinations=True):
     """
-    Generate the list of disturbing function terms for a 
-    j:j-k resonance with eccentricity/inclination order
-    between Nmin and Nmax.
+    Generate the list of disturbing function terms for a
+    p:p-q resonance with eccentricity/inclination order
+    between min_order and max_order
 
     Arguments
     ---------
-    j : int
-     Determines resonance
-    k : int
-     Order of the resonance
-    Nmin : int
-     Minimum order of terms to include
-    Nmax : int
-     Maximum order of terms to include
+    p : int
+        Determines resonance
+    q : int
+        Order of the resonance
+    min_order : int, optional
+        Minimum order in eccentricities and inclinations to include. Defaults to order of the resonance q (leading order)
+    max_order: int
+        Maximum order in eccentricities and inclinations to include. Defaults to order of the resonance q (leading order)
+    eccentricities: bool, optional
+        By default, includes all eccentricity terms.
+        Can set to False to exclude any eccentricity terms (e.g., fixed circular orbits).
+    inclinations: bool, optional
+        By default, includes all inclination terms.
+        Can set to False to exclude any inclination terms (e.g., co-planar systems).
 
     Returns
     -------
     term : list
-        A list of disturbing function terms. 
+        A list of disturbing function terms.
         Each entry in the list is of the form
-        (kvec, zvc)
+        (k_vec, nu_vec) (see PoincareHamiltonian.add_cosine_term in poincare.py)
     """
-    args_dict = DFArguments_dictionary(Nmax)
+    if not min_order:
+        min_order = q
+    if not max_order:
+        max_order = q
+    args_dict = df_arguments_dictionary(max_order)
     args = []
-    for N in range(Nmin,Nmax+1):
-        for k1 in range(k,N+1,k):
-            if (N-k1) % 2:
+    for N in range(min_order,max_order+1):
+        for q1 in range(q,N+1,q):
+            if (N-q1) % 2:
                 continue
-            j1 = (k1//k) * j
-            for N1 in range(k1,N+1,2):
-                ztot = (N-N1)//2
-                for arg in args_dict[N1][k1]:
-                    for zc in _zcombos_iter(ztot):
-                        js = (j1,k1 - j1,*arg)
-                        args.append((js,zc))
+            p1 = (q1//q) * p
+            for N1 in range(q1,N+1,2):
+                nutot = (N-N1)//2
+                for arg in args_dict[N1][q1]:
+                    for nu_vec in _nucombos(nutot):
+                        k_vec = (p1,q1 - p1,*arg)
+                        if inclinations == False and k_nu_depend_on_inclinations(k_vec, nu_vec) == True:
+                            continue
+                        if eccentricities == False and k_nu_depend_on_eccentricities(k_vec, nu_vec) == True:
+                            continue
+                        args.append((k_vec,nu_vec))
     return args
 
-def SecularTermsList(Nmin,Nmax):
+def list_secular_terms(min_order,max_order,eccentricities=True,inclinations=True):
     """
     Generate the list of secular disturbing function terms 
-    with eccentricity/inclination order between Nmin and Nmax.
+    with eccentricity/inclination order between min_order and max_order.
 
     Arguments
     ---------
-    j : int
-     Determines resonance
-    k : int
-     Order of the resonance
-    Nmin : int
-     Minimum order of terms to include
-    Nmax : int
-     Maximum order of terms to include
+    min_order : int
+     Minimum order in eccentricities and inclinations to include.
+    max_order : int
+     Maximum order in eccentricities and inclinations to include.
+    eccentricities: bool, optional
+        By default, includes all eccentricity terms.
+        Can set to False to exclude any eccentricity terms (e.g., fixed circular orbits).
+    inclinations: bool, optional
+        By default, includes all inclination terms.
+        Can set to False to exclude any inclination terms (e.g., co-planar systems).
 
     Returns
     -------
     term : list
         A list of disturbing function terms. 
         Each entry in the list is of the form
-        (kvec, zvc)
+        (k_vec, nu_vec) (see PoincareHamiltonian.add_cosine_term in poincare.py)
     """
-    args_dict = DFArguments_dictionary(Nmax)
+    args_dict = df_arguments_dictionary(max_order)
     args = []
-    Nmax1 = (Nmax//2) * 2 
-    Nmin1 = (Nmin//2) * 2 
+    Nmax1 = (max_order//2) * 2 
+    Nmin1 = (min_order//2) * 2 
     for N in range(0,Nmax1 + 1,2):
         argsN = args_dict[N][0]
-        ztot_min = max( (Nmin1 - N)//2 , 0)
-        ztot_max = (Nmax1 - N)//2 
-        for ztot in range(ztot_min,ztot_max + 1):
-            for zc in _zcombos_iter(ztot):
+        nutot_min = max( (Nmin1 - N)//2 , 0)
+        nutot_max = (Nmax1 - N)//2 
+        for nutot in range(nutot_min,nutot_max + 1):
+            for nu_vec in _nucombos(nutot):
                 for arg in argsN:
-                    js = (0,0,*arg)
-                    args.append((js,zc))
+                    k_vec = (0,0,*arg)
+                    if inclinations == False and k_nu_depend_on_inclinations(k_vec, nu_vec) == True:
+                        continue
+                    if eccentricities == False and k_nu_depend_on_eccentricities(k_vec, nu_vec) == True:
+                        continue
+                    args.append((k_vec,nu_vec))
     return args
 
 def laplace_b(s,j,n,alpha):
-    """
-    Calculates nth derivative with respect to a (alpha) of Laplace coefficient b_s^j(a).
+    r"""
+    Calculates :math:`n`th derivative with respect to :math:`\alpha` of Laplace coefficient :math:`b_s^j(\alpha)`.
     Uses recursion and scipy special functions. 
-    
+
     Arguments
     ---------
     s : float 
@@ -176,10 +316,15 @@ def laplace_b(s,j,n,alpha):
     j : int 
         integer parameter of Laplace coefficient. 
     n : int 
-        return nth derivative with respect to a of b_s^j(a)
-    a : float
-        semimajor axis ratio a1/a2 (alpha)
-    """    
+        Specify the :math:`n`th derivative with respect to :math:`alpha`. 
+    alpha : float
+        Semi-major axis ratio, :math:`\alpha=a_{in}/a_{out}`.
+
+    Returns
+    -------
+    float
+        The value of the coefficient.
+    """
     assert alpha>=0 and alpha<1, "alpha not in range [0,1): alpha={}".format(alpha)
     if j<0:
         return laplace_b(s,-j,n,alpha)
@@ -198,77 +343,121 @@ def laplace_b(s,j,n,alpha):
         )
     return 2 * poch(s,j) * alpha**j * hyp2f1(s,s+j,j+1,alpha**2)/ factorial(j)
 
-def eval_DFCoeff_dict(Coeff_dict,alpha):
+def evaluate_df_coefficient_dict(coeff_dict,alpha):
     r"""
     Evaluate a dictionary representing a sum
     of Laplace coefficient terms like those returned
-    by DFCoeff_C and DFCoeff_Cbar evaluated at a 
+    by df_coefficient_C and df_coefficient_Ctilde evaluated at a 
     specific value of semi-major axis ratio, alpha.
 
     Arguments
     ---------
-    Coeff_dict : dictionary
+    coeff_dict : dictionary
         Dictionary with entries {(p,(s,j,n)) : coeff}
         representing a sum of Laplace coefficients:
 
         .. math::
-         \mathrm{coeff}  \alpha^p \frac{ d^n b_s^{(j)}(\alpha)} { d\alpha^n}
+         \mathrm{coeff} \times \alpha^p \frac{ d^n b_s^{(j)}(\alpha)} { d\alpha^n}
     alpha : float
-        Value of semi-major axis ratio a1/a2 appearing
+        Value of semi-major axis ratio, :math:`\alpha=a_i/a_j`, appearing
         as an argument of Laplace coefficients.
 
     Returns
     -------
-    float : 
+    float :
         The sum of Laplace coefficeint terms represented
         by dictionary entries.
     """
     tot = 0
-    for key,val in Coeff_dict.items():
-        if key is 'indirect':
-            tot += val / np.sqrt(alpha)
+    for key,val in coeff_dict.items():
+        if key[0] == 'indirect':
+            pwer = key[1]
+            rt_alpha_inv = 1 / np.sqrt(alpha) 
+            tot += val * rt_alpha_inv**pwer
         else:
             p,arg = key
             tot += val * alpha**p * laplace_b(*arg,alpha)
     return tot
 
-def eccentricity_type_resonance_coefficient(j,k,l,alpha):
+
+def calB(n,k,p):
+    arglist = [negative_binom(p,l) * factorial(l) for l in range(1,n-k+3)]
+    return bell(n,k,arglist)
+
+def falling_factorial(x,n):
+    return poch(-x,n) * (-1)**n
+
+def _Psi_coeff(l1,l2,p1,p2,m1,m2,r1,r2):
+    return binom(l1,m1) * binom(l2,m2) *\
+    falling_factorial(-2*r1 - p2/2,m2) * falling_factorial(-p1/2,m1) *\
+    calB(l1-m1,r1,2) * calB(l2-m2,r2,-2)
+
+def _calc_df_coefficient_C_l1_l2_Taylor_coeff(l1,l2,p1,p2,derivs_list):
+    tot = 0
+    l1fact_inv = 1/factorial(l1)
+    l2fact_inv = 1/factorial(l2)
+    prefactor = l1fact_inv * l2fact_inv
+    for m1 in range(l1+1):
+        for r1 in range(l1-m1+1):
+            for m2 in range(l2+1):
+                for r2 in range(l2-m2+1):
+                    val = prefactor * derivs_list[r1+r2] * _Psi_coeff(l1,l2,p1,p2,m1,m2,r1,r2)
+                    tot=tot+val        
+    return tot
+
+def _p1_p2_from_k_nu(kvec,nuvec):
     r"""
-    Get the coefficient of the distrubing function term:
-        e_1^{l}e_2^{k-l}\cos[j \lambda_2 - (j-k) \lambda_1 - l\varpi_1 - (k-l)\varpi_2]
-    that appears as the leading-order term of a kth order eccentricity resonance.
+    Convenience function for calculate integers p1 and p2 
+    defined as 
+        p1 = abs(k3) + abs(k5) + 2 * nu1 + 2 * nu3
+        p2 = 4 + abs(k4) + abs(k6) + 2 * nu2 + 2 * nu4
+
+    These definitions appear in the formula for the expansion
+    of DF coefficients in powers delta
+    """
+    k1,k2,k3,k4,k5,k6 = kvec
+    nu1,nu2,nu3,nu4 = nuvec
+    p1 = abs(k3) + abs(k5) + 2 * nu1 + 2 * nu3
+    p2 = 4 + abs(k4) + abs(k6) + 2 * nu2 + 2 * nu4
+    return p1,p2
+
+def evaluate_df_coefficient_delta_expansion(Coeff_dict,p1,p2,lmax,alpha):
+    r"""
+    Calculate coefficients of the Taylor series of `Coeff_dict` in 
+    powers math:`\delta_i` and math::`\delta_j` evaluated at semi-major
+    axis ratio math:`\alpha=a_1/a_2`.
 
     Arguments
     ---------
-    j : int
-        Specifies the resonance term
-    k : int
-        Order of the resonance
-    l : int
-        Specify the e_1^{l}e_2^{k-l} sub-resonance
+    Coeff_dict : dict
+        Dictionary representation of DF coefficient
+    p1 : int
+        Integer specific to the DF term's (k,nu) values
+        given by
+            p1 = abs(k3) + abs(k5) + 2 * nu1 + 2 * nu3
+    p2 : int
+        Integer specific to the DF term's (k,nu) values
+        given by
+            p2 = 4 + abs(k4) + abs(k6) + 2 * nu2 + 2 * nu4
+    lmax : int
+        Maximum power of Taylor expansion.
     alpha : float
-        Semi-major axis ratio a_1/a_2
-
-    Returns
-    -------
-    val : float
-        Coefficient's numerical value
+        Semi-major axis ratio at which to evaluate coefficients.
     """
-    if l >  k:
-        raise ValueError("Integer arguemnt l={} cannot be greater than the resonance order k={}".format(l,k))
-    j1 = j
-    j2 = k - j
-    j3 = -l
-    j4 = l-k
-    j5 = 0
-    j6 = 0
-    z1=z2=z3=z4=0
-    coeff = DFCoeff_C(j1,j2,j3,j4,j5,j6,z1,z2,z3,z4)
-    ncoeff = eval_DFCoeff_dict(coeff,alpha)
-    return ncoeff
+    answer = dict()
+    C = Coeff_dict
+    # Derivatives of C w.r.t. alpha
+    NC_derivs = [evaluate_df_coefficient_dict(C,alpha)]
+    for ltot in range(lmax+1):
+        C=deriv_df_coefficient(C)
+        NC_derivs.append(alpha**(ltot+1) * evaluate_df_coefficient_dict(C,alpha))
+        for lIn in range(ltot+1):
+            lOut = ltot - lIn
+            answer[(lIn,lOut)] = _calc_df_coefficient_C_l1_l2_Taylor_coeff(lIn,lOut,p1,p2,NC_derivs)
+    return answer
 
 # Vector of resonance coefficients
-def get_res_coeff_vector(j,k):
+def get_res_coefficient_vector(j,k,include_indirect_terms=True):
     r"""
     Get a vector comprised of all sub-resonance coefficients for the j:j-k mean motion resonance.
 
@@ -279,9 +468,9 @@ def get_res_coeff_vector(j,k):
     k : int
         Order of the resonance
     include_indirect_terms :  boole, optional
-        Whether the contribution of indirect terms should be
-        accounted for when computing the coefficients. Default
-        is True.
+        whether the contribution of indirect terms should be
+        accounted for when computing the coefficients. default
+        is true.
 
     Returns
     -------
@@ -290,15 +479,16 @@ def get_res_coeff_vector(j,k):
     """ 
     res_pratio = float(j - k) /float(j)
     alpha = res_pratio**(2./3.)
-    Cjkl = eccentricity_type_resonance_coefficient
+    res_ecc_arg = lambda j,k,l: (j,k-j,-l,l-k,0,0,0,0,0,0) # k and nu values
+    Cjkl = lambda j,k,l,alpha: evaluate_df_coefficient_dict(df_coefficient_C(*res_ecc_arg(j,k,l),include_indirect_terms=include_indirect_terms),alpha)
     vals = np.array([Cjkl(j,k,l,alpha) for l in range(k+1)],dtype=np.float64)
     return vals
 
-def get_fg_coeffs(res_j,res_k):
+def get_fg_coefficients(res_j,res_k):
     """Get 'f' and 'g' coefficients for approximating the disturbing function coefficients associated with an MMR."""
     res_pratio = float(res_j - res_k) /float(res_j)
     alpha = res_pratio**(2./3.)
-    vec = get_res_coeff_vector(res_j,res_k)
+    vec = get_res_coefficient_vector(res_j,res_k)
     resids_vec_fn = lambda fg: vec - np.array([binomial(res_k,l) * fg[0]**(l) * fg[1]**(res_k-l) for l in range(res_k+1)],dtype=np.float64)
     ex = (1-alpha)
     f0 = -1 / ex
@@ -436,9 +626,9 @@ def threeFtwo(a,b):
 
 def KaulaF(n,q,p,j):
     """
-    Series coefficient in the Taylor expansion of the
-    Kaula inclination function F_{nqp}(I).
-    See, e.g., Kaula (1962,1966) or Ellis & Murray (2000).
+    Series coefficient in the Taylor expansion of the Kaula inclination
+    function F_{nqp}(I).  See, e.g., `Kaula (1962,1966)`_ or `Ellis & Murray
+    (2000)`_.
 
     The function returns the jth term of the Taylor
     expansion in the variable s = sin(I/2). I.e.
@@ -448,6 +638,8 @@ def KaulaF(n,q,p,j):
     package by Fabio Zugno available at:
         https://library.wolfram.com/infocenter/MathSource/4256/
 
+    .. _Kaula (1962,1966): https://ui.adsabs.harvard.edu/abs/1962AJ.....67..300K/abstract
+    .. _Ellis & Murray (2000): https://ui.adsabs.harvard.edu/abs/2000Icar..147..129E/abstract
     Arguments
     ---------
     n : int
@@ -478,6 +670,7 @@ def KK(i,n,m):
     numerator = (-1)**(i-n) * (1 + 2 * n) * gamma(1/2 + i) * gamma(3/2 + i)
     denom = 4 * gamma((2 + i-m-n)/2) * gamma((2 + i + m-n)/2) * gamma((3 + i - m + n)/2) * gamma((3 + i + m + n)/2)
     return numerator / denom
+
 def getrange(lim1,lim2,n=1):
     assert n>0, "Negative interval n={} passed to getrange".format(n)
     if lim1 < lim2:
@@ -512,133 +705,201 @@ def FX(h,k,i,p,u,v1,v2,v3,v4,z1,z2,z3,z4):
 
     
 
-def DFCoeff_Cbar(j1,j2,j3,j4,j5,j6,z1,z2,z3,z4,include_indirect = True):
+def df_coefficient_Ctilde(k1,k2,k3,k4,k5,k6,nu1,nu2,nu3,nu4,include_indirect = True):
     r"""
     Get the coefficient of the disturbing function term:
-    
-      s1^{|j5|+2z1} s2^{|j6|+2z2} * e2^{|j4|+2*z4} * e1^{|j3|+2*z3} \times 
-          cos[j1*L2 + j2*L1 + j3 * pomega1 + j4 * w2 + j5 * Omega1 + j6 * Omega2)
 
-    where s1 = sin(I1/2) and s2 = sin(I2/2) as a dictionary of Laplace coefficient 
-    arguemnts and their numerical coefficents.
-    
+    .. math::
+        s_i^{|k_5|+2\nu_1} s_j^{|k_6|+2\nu_2}
+        e_i^{|k_3|+2\nu_4} e_j^{|k_4|+2\nu_3} 
+        \times
+        \cos[
+            k_1\lambda_j + k_2\lambda_i +
+            k_3 \varpi_i + k_4 \varpi_j +
+            k_5 \Omega_i + k_6 \Omega_j
+        ]
+
+    where the indices :math:`i` and :math:`j` corresponds to the inner and
+    outer planets, respectively.  The result is returned as a dictionary of
+    Laplace coefficient arguemnts and their numerical coefficents.
+
     Arguments:
     ----------
-    j1 : int
+    k1 : int
         Coefficient of outer planet's mean longitude in cosine argument
-    j2 : int
+    k2 : int
         Coefficient of inner planet's mean longitude in cosine argument
-    j3 : int
+    k3 : int
         Coefficient of inner planet's mean longitude in cosine argument
-    j4 : int
+    k4 : int
         Coefficient of outer planet's mean longitude in cosine argument
-    j5 : int
+    k5 : int
         Coefficient of inner planet's longitude of ascending node in cosine argument
-    j6 : int
+    k6 : int
         Coefficient of outer planet's longitude of ascending node in cosine argument
-    z1 : int
-        Select specific term where the exponent of s1 is |j5|+2*z1
-    z2 : int
-        Select specific term where the exponent of s2 is |j6|+2*z3
-    z3 : int
-        Select specific term where the exponent of e1 is |j3|+2*z3
-    z4 : int
-        Select specific term where the exponent of e1 is |j4|+2*z4
+    nu1 : int
+        Select specific term where the exponent of s1 is |k5|+2*nu1
+    nu2 : int
+        Select specific term where the exponent of s2 is |k6|+2*nu3
+    nu3 : int
+        Select specific term where the exponent of e1 is |k3|+2*nu3
+    nu4 : int
+        Select specific term where the exponent of e1 is |k4|+2*nu4
     include_indirect : booole, optional
         Whether to include the indirect contribution to the disturibing function
         coefficient.
-        
+
     Returns
     -------
-    dictionary 
+    dictionary
         The coefficient is given by the sum over laplace coefficients
         contained in the dictionary entries: 
 
         ..math::
-            \sum C \times \alpha^p \frac{d^{n}}{d\alpha^{n}} b_{s}^{j}(\alpha)
-        where the dictionary entries are in the form { (p,(s,j,n)) : C }
+            \sum A \times \alpha^p \frac{d^{n}}{d\alpha^{n}} b_{s}^{j}(\alpha)
+
+        where the dictionary entries are in the form { (p,(s,j,n)) : A }
     """
     total = defaultdict(float)
     # must be even power in inclination
-    if abs(j5 + j6) % 2:
+    if abs(k5 + k6) % 2:
         warnings.warn(
-                "\n DFCoeff called with an argument not symmetric w.r.t. planet inclinations:\n" + 
-                "\t (j1,j2,j3,j4,j5,j6)=({},{},{},{},{},{})".format(j1,j2,j3,j4,j5,j6)
+                "\n df_coefficient called with an argument not symmetric w.r.t. planet inclinations:\n" + 
+                "\t (k1,k2,k3,k4,k5,k6)=({},{},{},{},{},{})".format(k1,k2,k3,k4,k5,k6)
                 )
         return dict(total)
     # Sum of integer coefficients must be 0
-    if j1 + j2 + j3 + j4 + j5 + j6:
+    if k1 + k2 + k3 + k4 + k5 + k6:
         warnings.warn(
-                "\n DFCoeff called with an argument that does not satisfy D'Alembert relation:\n" + 
-                "\t (j1,j2,j3,j4,j5,j6)=({},{},{},{},{},{})".format(j1,j2,j3,j4,j5,j6)
+                "\n df_coefficient called with an argument that does not satisfy D'Alembert relation:\n" + 
+                "\t (k1,k2,k3,k4,k5,k6)=({},{},{},{},{},{})".format(k1,k2,k3,k4,k5,k6)
                 )
         return dict(total)
-    jvec = np.array([j1,j2,j3,j4,j5,j6])
-    if np.alltrue(jvec==0):
-        for i in getrange(0,z1+z2,1):
+    kvec = np.array([k1,k2,k3,k4,k5,k6])
+    if np.alltrue(kvec==0):
+        for i in getrange(0,nu1+nu2,1):
             for p in getrange(i%2,i,2):
-                for u in getrange(0,2*z3+2*z4):
-                    cf = FX(0,0,i,p,u,0,0,0,0,z1,z2,z3,z4) * (1 + (p != 0))
+                for u in getrange(0,2*nu3+2*nu4):
+                    cf = FX(0,0,i,p,u,0,0,0,0,nu1,nu2,nu3,nu4) * (1 + (p != 0))
                     if not np.isclose(cf,0):
                         total[(i+u,(i+1/2,abs(p),u))]+=cf
-    elif np.alltrue(jvec[2:]==0):
-        j = abs(j1)
-        for i in getrange(0,z1+z2,1):
+    elif np.alltrue(kvec[2:]==0):
+        j = abs(k1)
+        for i in getrange(0,nu1+nu2,1):
             for p in getrange(i%2,i,2):
-                for u in getrange(0,2*z3+2*z4):
-                    cf = FX(0,0,i,p,u,j,j,j,j,z1,z2,z3,z4) 
+                for u in getrange(0,2*nu3+2*nu4):
+                    cf = FX(0,0,i,p,u,j,j,j,j,nu1,nu2,nu3,nu4) 
                     if not np.isclose(cf,0):
                         total[(i+u,(i+1/2,abs(j+p),u))]+=cf
                         if p != 0:
                             total[(i+u,(i+1/2,abs(j-p),u))]+=cf
     else:
-        h = (-j5-j6)//2
-        k = (j6-j5)//2
-        n0 = int(np.ceil(max(h,-j5/2,-j6/2))) 
-        for i in getrange(n0,z1 + z2 + (abs(j5)+abs(j6)//2),1):
+        h = (-k5-k6)//2
+        k = (k6-k5)//2
+        n0 = int(np.ceil(max(h,-k5/2,-k6/2))) 
+        for i in getrange(n0,nu1 + nu2 + (abs(k5)+abs(k6)//2),1):
             for p in getrange(h-i,i-h,2):
-                for u in getrange(0,2*z3 + 2*z4 + abs(j3) + abs(j4),1):
-                    cf = FX(h, k, i, p, u, j2 + j3, j2, j1 + j4, j1, z1, z2, z3, z4)
+                for u in getrange(0,2*nu3 + 2*nu4 + abs(k3) + abs(k4),1):
+                    cf = FX(h, k, i, p, u, k2 + k3, k2, k1 + k4, k1, nu1, nu2, nu3, nu4)
                     if not np.isclose(cf,0):
-                        total[(i+u,(i+1/2,abs(j1+j4-h+p),u))]+=cf
+                        total[(i+u,(i+1/2,abs(k1+k4-h+p),u))]+=cf
 
     # add indirect term
     if include_indirect:
-        total['indirect'] = DFCoeff_Cbar_indirect_piece(j1,j2,j3,j4,j5,j6,z1,z2,z3,z4)
+        coeff = df_coefficient_Ctilde_indirect_piece(k1,k2,k3,k4,k5,k6,nu1,nu2,nu3,nu4)
+        total[('indirect',1)] = coeff
     return dict(total)
 
-def DFCoeff_C(j1,j2,j3,j4,j5,j6,N1,N2,N3,N4):
+def _df_coeff_mult_by_alpha_power(coeffdict,r):
+    """
+    For a DF coefficient 'coeff' represented by 'coeffdict', 
+    return the new coefficient dictionary  alpha^r * coeff
+    """
+    new = dict()
+    for key,val in coeffdict.items():
+        if key[0]=='indirect':
+            newkey = (key[0],key[1]-2*r)
+        else:
+            p,sjn = key
+            newkey = (p+r,sjn)
+        new[newkey]=val
+    return new
+
+def _df_coefficient_scalar_mult(coeffdict,s):
+    """
+    Return dictionary for s * coeffdict for scalar value 's'
+    """
+    return {key:s*val for key,val in coeffdict.items()}
+
+def _get_alpha_times_derivs_list(coeffdict, ltot):
+    derivs_list=[coeffdict]
+    dcoeff = coeffdict.copy()
+    for l in range(ltot):
+        dcoeff = deriv_df_coefficient(dcoeff)
+        derivs_list.append(_df_coeff_mult_by_alpha_power(dcoeff,l+1))
+    return derivs_list
+
+
+
+def df_coefficient_C(k1,k2,k3,k4,k5,k6,nu1,nu2,nu3,nu4,l1=0,l2=0,include_indirect_terms = True):
     r"""
-    Get the coefficient of the disturbing function term:
+    Get the coefficient of the disturbing function term
 
-    Y1^{|j5|+2*N1} * Y2^{|j6|+2*N2} * X1^{|j3|+2*N3} * X2^{|j4|+2*N4}
-     *cos[j1*L2 + j2*L1 + j3 * pomega1 + j4 * w2 + j5 * Omega1 + j6 * Omega2)
+    .. math ::
 
-    as a dictionary of Laplace coefficient
+        |Y_i|^{|k_5|+2\nu_1}
+        |Y_j|^{|k_6|+2\nu_2}
+        |X_i|^{|k_3|+2\nu_3}
+        |X_j|^{|k_4|+2\nu_4}
+        \delta_i^{l_1}
+        \delta_j^{l_2}
+        \times \cos[
+            k_1 \lambda_j + k_2 \lambda_i +
+            k_3 \varpi_i + k_4 \varpi_j +
+            k_5 \Omega_i + k_6 \Omega_j
+        ]~,
+
+    where the indices :math:`i` and :math:`j` corresponds to the
+    inner and outer planets, respectively.
+    The result it returned as a dictionary of Laplace coefficient
     arguemnts and their numerical coefficents.
 
     Arguments:
     ----------
-    j1 : int
+    k1 : int
         Coefficient of outer planet's mean longitude in cosine argument
-    j2 : int
+    k2 : int
         Coefficient of inner planet's mean longitude in cosine argument
-    j3 : int
+    k3 : int
         Coefficient of inner planet's mean longitude in cosine argument
-    j4 : int
+    k4 : int
         Coefficient of outer planet's mean longitude in cosine argument
-    j5 : int
+    k5 : int
         Coefficient of inner planet's longitude of ascending node in cosine argument
-    j6 : int
+    k6 : int
         Coefficient of outer planet's longitude of ascending node in cosine argument
-    N1 : int
-        Select specific term where the exponent of Y1 is |j5|+2*N1
-    N2 : int
-        Select specific term where the exponent of Y2 is |j6|+2*N3
-    N3 : int
-        Select specific term where the exponent of X1 is |j3|+2*N3
-    N4 : int
-        Select specific term where the exponent of X2 is |j4|+2*N4
+    nu1 : int
+        Select specific term where the exponent of Y1 is |k5|+2*nu1
+    nu2 : int
+        Select specific term where the exponent of Y2 is |k6|+2*nu3
+    nu3 : int
+        Select specific term where the exponent of X1 is |k3|+2*nu3
+    nu4 : int
+        Select specific term where the exponent of X2 is |k4|+2*nu4 
+    l1 : int, optional
+        Select specifc term where exponent of 
+        :math:`\delta_1 = (\Lambda_1 - \Lambda_{1,0})/\Lambda_{1,0})`
+        is l1.
+        Default value is l1 = 0
+    l2 : int, optional
+        Select specifc term where exponent of 
+        :math:`\delta_2 = (\Lambda_2 - \Lambda_{2,0})/\Lambda_{2,0})`
+        is l2.
+        Default value is l2 = 0
+    include_indirect_terms :  boole, optional
+        whether the contribution of indirect terms should be
+        accounted for when computing the coefficients. default
+        is true.
 
     Returns
     -------
@@ -651,27 +912,45 @@ def DFCoeff_C(j1,j2,j3,j4,j5,j6,N1,N2,N3,N4):
 
         where the dictionary entries are in the form { (p,(s,j,n)) : C }
     """
-    terms_total = defaultdict(float)
-    for n3 in range(N3+1):
-        for n4 in range(N4+1):
-            term_dict = DFCoeff_Cbar(j1,j2,j3,j4,j5,j6,N1,N2,n3,n4)
-            prefactor = Xi(N3-n3,n3+abs(j3)/2,N1+abs(j5)/2) * Xi(N4-n4,n4+abs(j4)/2,N2+abs(j6)/2)
+    Chat = defaultdict(float)
+    msg = "Integer arguments nu_i and l_i must be non-negative."
+    assert np.alltrue(np.array([nu1,nu2,nu3,nu4,l1,l2])>=0), msg
+    for n3 in range(nu3+1):
+        for n4 in range(nu4+1):
+            term_dict = df_coefficient_Ctilde(k1,k2,k3,k4,k5,k6,nu1,nu2,n3,n4,include_indirect_terms)
+            prefactor = Xi(nu3-n3,n3+abs(k3)/2,nu1+abs(k5)/2) * Xi(nu4-n4,n4+abs(k4)/2,nu2+abs(k6)/2)
             if prefactor != 0.:
                 for key,val in term_dict.items():
-                    terms_total[key] += prefactor * val
-    return dict(terms_total)
+                    Chat[key] += prefactor * val
+    ltot = l1 + l2
+    if ltot==0:
+        result = Chat
+    else:
+        result = defaultdict(float)
+        derivs_list = _get_alpha_times_derivs_list(Chat,ltot)
+        l1fact_inv = 1/factorial(l1)
+        l2fact_inv = 1/factorial(l2)
+        prefactor = l1fact_inv * l2fact_inv
+        p1,p2= _p1_p2_from_k_nu([k1,k2,k3,k4,k5,k6],[nu1,nu2,nu3,nu4])
+        for m1 in range(l1+1):
+            for r1 in range(l1-m1+1):
+                for m2 in range(l2+1):
+                    for r2 in range(l2-m2+1):
+                        to_add = _df_coefficient_scalar_mult(derivs_list[r1+r2],prefactor * _Psi_coeff(l1,l2,p1,p2,m1,m2,r1,r2))
+                        result = _add_dicts(result,to_add)      
+    return result
 
-def has_indirect_component(j1,j2,j3,j4,j5,j6):
-    two_p = j2 + j4 + 1 
-    two_p1 = -1 * (j1 + j3 - 1)
-    m = j5 - two_p1 + 1
+def has_indirect_component(k1,k2,k3,k4,k5,k6):
+    two_p = k2 + k4 + 1 
+    two_p1 = -1 * (k1 + k3 - 1)
+    m = k5 - two_p1 + 1
     m_is_zero_or_one = m == 0 or m == 1
     return is_zero_or_two(two_p) and is_zero_or_two(two_p1) and m_is_zero_or_one
 
 def is_zero_or_two(n):
     return n == 0 or n == 2
     
-def deriv_DFCoeff(coeff):
+def deriv_df_coefficient(coeff):
     r"""
     Derivative of a disturbing function coefficient with repsect to 
     alpha.
@@ -694,13 +973,16 @@ def deriv_DFCoeff(coeff):
     """
     dcoeff = defaultdict(float)
     for key,val in coeff.items():
-        if key=='indirect':
-            # Fix!
-            continue
-        p,sjn= key
-        s,j,n = sjn
-        dcoeff[(p-1,sjn)] += p * val
-        dcoeff[(p,(s,j,n+1))] += val
+        if key[0]=='indirect':
+            # term \propto \alpha^{-p/2}
+            pwer = key[1]
+            dcoeff[('indirect',pwer + 2)] = -0.5 * pwer * val
+        else:
+            p,sjn= key
+            s,j,n = sjn
+            if p>0:
+                dcoeff[(p-1,sjn)] += p * val
+            dcoeff[(p,(s,j,n+1))] += val
     return dict(dcoeff)
 
 def Xi(N,p,q):
@@ -718,7 +1000,7 @@ def negative_binom(minus_q,l):
 
 
 def _Cindirect_type1(k1,k2,m,z1,z2,z3,z4):
-    """
+    r"""
     Indirect term cosine coefficient for argument:
          \theta = k_1\lambda_j + k_2\lambda_i
                     - (k_1-1)\varpi_j - (k_2+1) \varpi_i
@@ -755,7 +1037,7 @@ def _Cindirect_type2(k1,k2,m,z1,z2,z3,z4):
         return binom(1 ,z2) * (1 - _delta(z1)) * (-1)**(z2) * base_case
 
 
-def DFCoeff_Cbar_indirect_piece(k1,k2,k3,k4,k5,k6,z1,z2,z3,z4):
+def df_coefficient_Ctilde_indirect_piece(k1,k2,k3,k4,k5,k6,z1,z2,z3,z4):
     r"""
     Get the indirect contribution to disturbing function coefficient
 
@@ -764,7 +1046,7 @@ def DFCoeff_Cbar_indirect_piece(k1,k2,k3,k4,k5,k6,z1,z2,z3,z4):
     """
     if np.sum([k1,k2,k3,k4,k5,k6]) !=0:
         warnings.warn(
-        "\n DFCoeff called with an argument that does not satisfy D'Alembert relation:\n" +
+        "\n df_coefficient called with an argument that does not satisfy D'Alembert relation:\n" +
         "\t (k1,k2,k3,k4,k5,k6)=({},{},{},{},{},{})".format(k1,k2,k3,k4,k5,k6)
         )
         return 0
@@ -780,6 +1062,7 @@ def DFCoeff_Cbar_indirect_piece(k1,k2,k3,k4,k5,k6,z1,z2,z3,z4):
     else:
         return 0
 
+# SH --- Needs modified to accomodate expansion in delta.
 def terms_list_to_HamiltonianCoefficients_dict(terms_list,G,mIn,mOut,MIn,MOut,Lambda0In,Lambda0Out,include_alpha_derivs=False):
     """
     Retrieve the a dictionary with the coefficient values of terms appearing in the Hamiltonian.
@@ -837,12 +1120,12 @@ def terms_list_to_HamiltonianCoefficients_dict(terms_list,G,mIn,mOut,MIn,MOut,La
     prefactor = -G * mIn * mOut * aOut_inv
     result = dict()
     for kvec,zvec in terms_list:
-        C = DFCoeff_C(*kvec,*zvec)
-        coeff = prefactor * eval_DFCoeff_dict(C,alpha)
+        C = df_coefficient_C(*kvec,*zvec)
+        coeff = prefactor * evaluate_df_coefficient_dict(C,alpha)
         if include_alpha_derivs:
-            ind = C.pop('indirect')
-            dC = deriv_DFCoeff(C)
-            dcoeff = prefactor * ( eval_DFCoeff_dict(dC,alpha) - 0.5 * ind / np.sqrt(alpha*alpha*alpha))
+            ind = C.pop(('indirect',1))
+            dC = deriv_df_coefficient(C)
+            dcoeff = prefactor * ( evaluate_df_coefficient_dict(dC,alpha) - 0.5 * ind / np.sqrt(alpha*alpha*alpha))
             result[(kvec,zvec)] = coeff,dcoeff
         else:
             result[(kvec,zvec)] = coeff
@@ -1037,16 +1320,16 @@ def _add_dicts(*dicts):
 
 
 def _resonance_arguments_of_fixed_order(j,k,N):
-    args_dict = DFArguments_dictionary(N)
+    args_dict = df_arguments_dictionary(N)
     args = []
     for N1 in range(k,N+1,2):
         if (N-N1) % 2:
             continue
-        ztot = (N-N1)//2
+        nutot = (N-N1)//2
         for arg in args_dict[N1][k]:
-            for zc in _zcombos_iter(ztot):
+            for nuc in _nucombos(nutot):
                 js = (j,k - j,*arg)
-                args.append((js,zc))
+                args.append((js,nuc))
     return args
 
 def resonant_secular_contribution_dictionary(j,k,Nmin,Nmax,G,mIn,mOut,MIn,MOut,Lambda0In,Lambda0Out):
